import React from 'react';
import MuiIcon from '@material-ui/core/Icon';
import { createMuiTheme } from '@material-ui/core';
import { ThemeProvider } from '@material-ui/styles';
import PropTypes from 'prop-types';

// Styles
import getStyles from './Icon.styles';
import './style.css';

/** Default icon size. */
const ICON_SIZE_DEFAULT = 'default';

/**
 * Icon  component :allows a user to view a non-image icon.
 * @see https://material-ui.com/style/icons/
 */
const Icon = (props) => {
    const {
        contextStyles,
        iconName,
        title,
        iconSize,
        testID,
    } = props;

    const styles = getStyles(contextStyles);

    const theme = createMuiTheme({
        typography: {
            // https://material-ui.com/style/typography/#migration-to-typography-v2
            useNextVariants: true,
        },
        overrides: {
            MuiIcon: { // Name of the component ⚛️ / style sheet
                root: {// Name of the rule
                    ...styles.icon,
                },
            },
        },
    });

    let iconTitle = title;
    if (!iconTitle) {
        iconTitle = iconName;
    }

    if (iconName) {
        return (
            <div
                style={ styles.container }
                className="xha-icon"
                data-testid={ testID && `${testID}__container` }>

                <ThemeProvider theme={ theme }>
                    <MuiIcon
                        fontSize={ iconSize }
                        title={ iconTitle }
                        data-testid={ testID && `${testID}__icon` }>

                        { iconName }

                    </MuiIcon>
                </ThemeProvider>
            </div>
        );
    }

    return null;
};

Icon.propTypes = {

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the icon */
        container: PropTypes.object,

        /** Styles for a icon */
        icon: PropTypes.object,
    }),

    /**
     * Name of a XHA UI icon, or an icon element, to render.
     * This prop is required because it determines the content of the component, but it can
     * be explicitly set to falsy values to render nothing.     *
     * If `null` or `undefined` or `false`, this component will render nothing.
     */
    iconName: PropTypes.string.isRequired,

    /**
     * Description string.
     * Browsers usually render this as a tooltip on hover, whereas screen
     * readers will use it for aural feedback.
     * By default, this is set to the icon's name for accessibility.
     */
    title: PropTypes.string,

    /**
     * Size of the icon to be rendered.
     */
    iconSize: PropTypes.oneOf([
        'inherit',
        'default',
        'small',
        'large',
    ]),

    /** Id used for testing */
    testID: PropTypes.string,
};

Icon.defaultProps = {
    contextStyles: {},
    title: '',
    iconSize: ICON_SIZE_DEFAULT,
};

export default Icon;
